/*--------------------------------------------------------------------------
 * Copyright (C) 1992 - 1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  EpromCpy.C - This utility uses TMEX to copy a file of hex data into an EPROM 
 *               1-Wire device such as a DS2502/DS2505.
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.03
 *
 */

#define TMEXUTIL

#include "iBTMEX.H"   

/* constants */
#define MEMTYPE_EPROM1         2
#define MEMTYPE_EPROM2         3
#define MEMTYPE_EPROM3         4

#define MAX_WRITEBUF           1024
#define MAX_FILEBUF            5000

#define START_ADDRESS          0x00

#define USE_BLOCK_MODE         1  

/* Local Function Prototypes */
short FindFirstEPROMDevice(long);
void Exit_Prog(char *, short);
short DumpDevice(void);
short ToHex(char);
short iButtonFound(void);
short ParseData(char *, short, uchar *, short);
short ProgramBlock(ushort, ushort, uchar *, short);
short VerifyBlock(ushort, ushort, uchar *, short);

/* Global Variables to hold list information */
short PortType, PortNum;
uchar state_buffer[5120];
ushort CRC16;
short ROM[9];

/*----------------------------------------------------------------------
 *  This is the Main routine for EpromCpy
 */
void main(short argc, char **argv)
{
   short writelen=0,i,fllen,loopcnt=0;
   char tstr[80];       
   char filebuf[MAX_FILEBUF];
   uchar writebuf[MAX_WRITEBUF];
   FILE *fp;

   /* check arguments to see if request instruction with '?' or too many */
   if ((argc != 2) || ((argc == 2) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: EpromCpy InputFile\n"
              "  - argument 1 specifies the InputFile to write to EPROM\n"
              "  - version 1.03\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default PortNum and PortType */
   if (!TMReadDefaultPort(&PortNum,&PortType))
       Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* print a header */  
   printf("\nEpromCpy Version 1.03\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-1999\n\n");
   printf("Default port number: %d    port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("TMEX main driver: %s\n",tstr);
   printf("TMEX adapter driver, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n",tstr);

   #ifdef USE_BLOCK_MODE
     printf("USING BLOCK MODE\n");
   #else
     printf("USING byte MODE\n");
   #endif

   /* open the input file and read it */
   fp=fopen(argv[1],"r");
   if(fp != NULL)
   {
      /* read the file */
      fllen = fread(filebuf,1,MAX_FILEBUF,fp);
      /* close the file */
      fclose(fp);
      /* attempt to convert the text into an array of uchars */
      writelen = ParseData(filebuf, fllen, writebuf, MAX_WRITEBUF);
   }

   /* check to verify the file data is correct */ 
   if (writelen == 0)
      Exit_Prog("InputFile not found or data is not in the correct format!\n",BAD_ARGUMENT);

   /* print out the buffer to write */ 
   printf("\n\nThe data to write starting at address %04X:\n",START_ADDRESS);   
   for (i = 0; i < writelen; i++)
   {
      printf(" %02X ",writebuf[i]);
      if (((i + 1) % 16) == 0)
         printf("\n");
   }
   printf("\n\n");

   /* find a device */
   if (iButtonFound())                          
   {   
      do 
      {
         /* write the buffer */
         if (ProgramBlock(START_ADDRESS, 0, writebuf, writelen))
         {
            printf("EPROM write successful, read to verify\n");
            if (VerifyBlock(START_ADDRESS,0,writebuf,writelen))
               Exit_Prog("\nEPROM write verified correct!\n\n", NORMAL_EXIT);
            else if (loopcnt < 10)
               printf("Verify was incorrect so try to write again, retry %d\n",loopcnt + 1);
            else
               Exit_Prog("Verify was incorrect, failed to write device!\n",-FILE_WRITE_ERR);
         }
         else
            printf("Error during write, retry %d\n",loopcnt + 1);
      }
      while (loopcnt++ < 10);
   }   
   else
      Exit_Prog("\n",-NO_DEVICE);

   Exit_Prog("Exceeded the number of retries, failed to write device!\n",-FILE_WRITE_ERR);
}

/*----------------------------------------------------------------------
 * Enter a hex string and convert it to an unsigned short
 */                                                          
short iButtonFound(void)
{
   short printmsg=1,found;
   long session_handle;                         

   /* loop to get a session handle find an EPROM device */
   found = FALSE;
   for (;;)
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         /* setup the MicroLan */
         if (TMSetup(session_handle) == 1)
         {
            printf("Searching for an EPROM iButton...\n\n");
            
            /* look for the first EPROM iButton */
            if (FindFirstEPROMDevice(session_handle))
            {
               found = TRUE;
               break;
            }
            /* iButton not on MicroLan */
            else   
            {
               printf("EPROM iButton not found on MicroLan!"); 
               break;
            }
         }   
         /* MicroLan port not valid */
         else   
         {
            printf("MicroLan setup failed!"); 
            break;
         }
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   /* end the session if one is open */
   TMEndSession(session_handle);

   return found;
}

/*----------------------------------------------------------------------
 * Parse the raw file data in to an array of uchar's.  The data must
 * be in the format of dual hex characters seperated by white space.
 * For example:
 *     12 34 45 67 89 AB CD EF
 *     FE DC BA 98 76 54 32 10
 * would be converted to an array of 16 uchars.
 * return the array length.  If an invalid hex pair is found then the
 * return will be 0.
 */                                                          
short ParseData(char *inbuf, short insize, uchar *outbuf, short maxsize)
{
   short ps, outlen=0;

   /* loop until end of file */
   for (ps = 0; ps < insize; ps++)
   { 
      /* check for white space */
      if (isspace(inbuf[ps]))
         continue;
      /* make sure there is 1 more char after this one */
      else if ((ps + 1) < insize)
      {
         /* make sure next two chars are hex */
         if (isxdigit(inbuf[ps]) && isxdigit(inbuf[ps+1]))
         {
            outbuf[outlen++] = (ToHex(inbuf[ps]) << 4) | ToHex(inbuf[ps+1]);
            ps++;         
         }  
         else
            return 0;   
      }
      else
         return 0;   

      // if getting to the max return what we have
      if ((outlen + 1) >= maxsize)
         return outlen;
   }

   return outlen;
}

/*----------------------------------------------------------------------
 * Program and verify the specified byte
 */                                                          
short ProgramBlock(ushort start_address, ushort pagetype, 
                                uchar *writebuf, short writelen)
{
   short printmsg=1,flag,rt=0,cnt=0,bits,goterror=0;
   long session_handle;    
   short i; 

   // loop to get a session handle 
   do
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         #ifdef USE_BLOCK_MODE
            /* call the program block function */ 
            bits = 8;
            flag = TMProgramBlock(session_handle,&state_buffer,writebuf,writelen,
                                  start_address,&bits);
            if (flag != writelen)
            {
               goterror = TRUE;
               printf("[rslt:%d]\n",flag);
               // check for no programming ability 
               if (flag == -3)
                  Exit_Prog("\nEPROM programming not available with this adapter.\n",-FUNC_NOT_SUP);
               // check for program not possible 
               if (flag == -11)
                  Exit_Prog("\nEPROM found is not empty and program is not possible!\n",-PROGRAM_WRITE_PROTECT);
            }
         #else
            // loop to write each byte 
            bits = 8;
            for (i = 0; i < writelen; i++)
            {
               // program the byte 
               flag = TMProgramByte(session_handle,&state_buffer,writebuf[i],
                                (short)(i + start_address),pagetype,&bits,0);
               if (flag != 1)
               {
                  goterror = TRUE;
                  printf("[rslt:%d, addr:%04X, data:%02X]\n",flag,i+start_address,writebuf[i]);
                  // check for no programming ability 
                  if (flag == -3)
                     Exit_Prog("\nEPROM programming not available with this adapter.\n",-FUNC_NOT_SUP);
                  // check for program not possible 
                  if (flag == -11)
                     Exit_Prog("\nEPROM found is not empty and program is not possible!\n",-PROGRAM_WRITE_PROTECT);
               }
            }         
         #endif

         /* end the session if one is open */
         TMEndSession(session_handle);
         break;
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
      
      Sleep(20);
   }
   while (cnt++ < 10);

   return !goterror;                  
}


/*----------------------------------------------------------------------
 * Read and verify that the data that was written is in the device.
 */                                                          
short VerifyBlock(ushort start_address, ushort pagetype, 
                                uchar *writebuf, short writelen)
{    
   short p,d,printmsg=1,flag,rt=0,cnt=0,done=0;
   short verified=0,firstpage,lastpage;
   long session_handle;    
   uchar pgbuf[50];                     

   /* calculate the first and last page need to verify this block */
   firstpage = start_address / 32;
   lastpage = (start_address + writelen) / 32;

   /* loop to get a session handle */
   do
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         /* assume will be correct, let compare invalidate if not */
         verified = TRUE;
         /* loop to read all needed pages to verify */
         for (p = firstpage; p <= lastpage; p++)
         {
            /* read the page */
            flag = TMExtendedReadPage(session_handle,&state_buffer,p,pgbuf,0);

            /* loop to verify each byte in the range */
            if (flag > 0)
            {
               for (d = 0; d < flag; d++)
               {
                  /* before or after data */
                  if (((p * 32 + d) < start_address) ||
                     ((p * 32 + d) >= (start_address + writelen)))
                     continue;
                  /* in data */
                  else
                  {
                     if (writebuf[(p * 32 + d) - start_address] == pgbuf[d])
                        printf(" %02X ",pgbuf[d]);
                     else
                     {
                        verified = FALSE;
                        printf("<%02X>",pgbuf[d]);
                     }

                     if ((((p * 32 + d) - start_address + 1) % 16) == 0)
                        printf("\n");
                  }
               }
            }
         }

         /* end the session if one is open */
         TMEndSession(session_handle);

         /* done with session */
         break; 
      }
      else if (session_handle == 0)
      {                            
         /* check if need to print the waiting message */
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the MicroLan\n\n");
         }
      }
      Sleep(20);
   }
   while (cnt++ < 10);

   /* return result of verification */
   return verified;                  
}

/*----------------------------------------------------------------------
 * Find the first EPROM type device
 */
short FindFirstEPROMDevice(long session_handle)
{
   short j,rt=0;

   /* loop to find EPROM device */   
   rt = TMFirst(session_handle,state_buffer);

   do
   {
      /* check on family type */
      if (rt >= 1)
      {
         /* read the rom number */
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   

         /* check if correct type */
         if ((FamSpec[ROM[0] & 0x7F].memtype == MEMTYPE_EPROM1) ||
             (FamSpec[ROM[0] & 0x7F].memtype == MEMTYPE_EPROM2) ||
             (FamSpec[ROM[0] & 0x7F].memtype == MEMTYPE_EPROM3))  
         {
            printf("Serial ROM ID: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }

      rt = TMNext(session_handle,state_buffer); 
   } 
   while (rt >= 1);
     
   /* failed to find device of that type */
   return 0;                              
}

/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);
   
   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   /*  return error code passed */
   exit(errcd);
}

//------------------------------------------------------------------------
// Convert 1 hex character to binary.
//
short ToHex(char ch)
{
   if ((ch >= '0') && (ch <= '9'))
      return ch - 0x30;
   else if ((ch >= 'A') && (ch <= 'F'))
      return ch - 0x37;
   else if ((ch >= 'a') && (ch <= 'f'))
      return ch - 0x57;
   else
      return 0;
}
                           
